ONTONS = "http://linked.earth/ontology#"
NSURL = "http://linked.earth/lipd"
DATAURL = "https://data.mint.isi.edu/files/lipd"

ARCHIVEURL = "http://linked.earth/ontology/archive"
PROXYURL = "http://linked.earth/ontology/proxy"
UNITSURL = "http://linked.earth/ontology/units"
VARIABLEURL = "http://linked.earth/ontology/variables"

DEFAULT_GRAPH_URI = "http://www.openrdf.org/schema/sesame#nil"

NAMESPACES = {
    "wgs84": "http://www.w3.org/2003/01/geo/wgs84_pos#",
    "le_archive": ARCHIVEURL,
    "le_proxy": PROXYURL,
    "le_units": UNITSURL,
    "le_variables": VARIABLEURL
}

